######################################################################
#
#  View User Record
#
######################################################################
#
#  This file is part of EPrints 2.
#  
#  EPrints 2 is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  EPrints 2 is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with EPrints 2; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
######################################################################

package EPrints::UserPage;

use EPrints::Session;
use EPrints::SearchExpression;
use EPrints::Utils;
use EPrints::User;

sub user_from_param
{
	my( $session ) = @_;

	my $username = $session->param( "username" );
	my $userid = $session->param( "userid" );

	if( !EPrints::Utils::is_set( $username ) && !EPrints::Utils::is_set( $userid ) )
	{
		$session->render_error( $session->html_phrase( 
				"lib/userpage:no_user" ) );
		return;
	}
	my $user;
	if( EPrints::Utils::is_set( $username ) )
	{
		$user = EPrints::User::user_with_username( $session, $username );
	}
	else
	{
		$user = EPrints::User->new( $session, $userid );
	}


	if( !defined $user )
	{
		$session->render_error( $session->html_phrase( 
				"lib/userpage:unknown_user" ) );
		return;
	}

	return $user;
}

sub process
{
	my( $session, $staff ) = @_;

	my $user = EPrints::UserPage::user_from_param( $session );
	return unless( defined $user );
	
	$userid = $user->get_value( "userid" );

	my( $page );

	$page = $session->make_doc_fragment();

	if( $staff )
	{	
		$page->appendChild( $user->render_full() );
	}
	else
	{
		$page->appendChild( $user->render() );
	}

	$page->appendChild( $session->render_ruler() );

	my $arc_ds = $session->get_archive()->get_dataset( "archive" );
	my $searchexp = new EPrints::SearchExpression(
		session => $session,
		dataset => $arc_ds );

	$searchexp->add_field(
		$arc_ds->get_field( "userid" ),
		$userid );

	$searchexp->perform_search();
	my $count = $searchexp->count();
	$searchexp->dispose();

	my $url;
	if( $staff )
	{
		$url = $session->get_archive()->get_conf( "perl_url" )."/users/staff/eprint_search?userid=$userid&_action_search=1";
	}
	else
	{
		$url = $session->get_archive()->get_conf( "perl_url" )."/user_eprints?userid=$userid";
	}
	my $link = $session->render_link( $url );	

	$page->appendChild( $session->html_phrase( 
				"lib/userpage:number_of_records",
				n=>$session->make_text( $count ),
				link=>$link ) );

	if( $staff && $session->current_user()->has_priv( "edit-user" ) )
	{
		$page->appendChild( $session->render_input_form(
			# no input fields so no need for a default
			buttons=>{
				_order => [ "edit", "delete" ],
				edit=>$session->phrase( "lib/userpage:action_edit" ),
				delete=>$session->phrase( "lib/userpage:action_delete" )
			},
			hidden_fields=>{
				userid=>$user->get_value( "userid" )
			},
			dest=>"edit_user"
		) );			
	}	
	

	$session->build_page(
		$session->html_phrase( "lib/userpage:title",
				name=>$user->render_description() ), 
		$page );
	$session->send_page();
}


1;
